; patch for NSC800 operation in Model 100
; lay this on top of base patch

; NSC800 Interrupt mask
; OUT 0BBh
; bit3 = 1, /RSTA enabled, =0 disabled
; bit2 = 1, /RSTB enabled, =0 disabled
; bit1 = 1, /RSTC enabled, =0 disabled
; bit0 = 1, /INTR enabled, =0 disabled

; after reset, bit 0 = 1, bit 1,2,3 = 0
; must use OUT (C),r or OUT (N),a  (same as OUT n in 8085)
	
;--------------------------------------------------------------------------------------
; free to use 0040h-007Fh -  move original "Basic function vector table for SGN to MID$" table 
; to hole at 75B4h

	.org	0040h
;--------------------------------------------------------------------------------------	
; SIM patch for 0D 

; 1  1  0  1 -- mask 7.5 and 5.5, enable 6.5
;    0  1  0  0 -- mask /RSTA /RSTC /INTR , enable /RSTB 

patch0D:
	mvi	a, 00000100b
	out	0BBh
	mvi	a,0Dh
	ret	
	

;--------------------------------------------------------------------------------------	
; SIM patch for 09 
; 1  0  0  1 -- mask set enable, enable 7.5 6.5, disable mask 5.5
;    1  1  0  0 -- enable /RSTA, /RSTB , disable /RSTC /INTR

; NOTE:  timer pulse needed to be modified in hardware since
; in 8085 the timer drives a flip flop and is not level detected.
; this means that the timer interrupt can be safely re-enabled in 8085 since no 
; more edges arrive.  In NSC800 the /RSTA input is level driven and so when enabled
; there is an immediate new interrupt.  Changing to a pulse fixes this.

patch09:

	mvi	a, 00001100b
	out	0BBh
	mvi	a,09h
	ret	
	
;--------------------------------------------------------------------------------------	
; SIM patch for 1D 

; 1  1  0  1 -- mask 7.5 and 5.5, enable 6.5
;    0  1  0  0 -- mask /RSTA /RSTC /INTR , enable /RSTB 

patch1D:
	mvi	a, 00000100b
	out	0BBh
	mvi	a,1Dh
	ret	
	
;--------------------------------------------------------------------------------------	

	.db	               00,00,00
	.db	00,00,00,00,00,00,00,00
	.db	00,00,00,00,00,00
;--------------------------------------------------------------------------------------
; NSC800 patch 1
; place copy of TRAP routine here for NSC800
	.org 	0066h
	JMP 	0F602H      	; RAM vector for TRAP interrupt	(TRAP) Handles power-down. 
				; User interface via {JUMP F602}
				; TRAP Power down

	.db	   00,00,00,00,00,00,00
	.db	00,00,00,00,00,00,00,00
	.db	00,00,00,00,00,00,00,00
	
		
;--------------------------------------------------------------------------------------
	.org	0102Fh
	.dw	basfun		; correct table pointer	

;--------------------------------------------------------------------------------------
; SIM patch for RST7.5 routine

; enters with interrupts disabled
; 1B39H	(3EH)	MVI A,0DH	; call routine to set mask 
; 1B3BH	(30H)	SIM
; 1B3CH	(FBH)	EI

; 1  1  0  1 -- mask set enable, mask 7.5 and 5.5, enable 6.5

	.org	01B39h
	call	patch0D

	
;--------------------------------------------------------------------------------------
; SIM patch for character plotting
; 457AH	(F3H)	DI
; 457BH	(3EH)	MVI A,0DH
; 457DH	(30H)	SIM
; 457EH	(FBH)	EI

; 1  1  0  1 -- mask set enable, mask 7.5 and 5.5, enable 6.5
	.org	0457Bh
	call	patch0D

	
;--------------------------------------------------------------------------------------
; SIM patch for character plotting
; 457FH	(CDH)	CALL 73A9H     ; Blink the cursor
; 4582H	(3EH)	MVI A,09H
; 4584H	(30H)	SIM

; 1  0  0  1 -- mask set enable, mask 5.5, enable 7.5 6.5

	.org	04582h
	call 	patch09


	
;--------------------------------------------------------------------------------------
	.org	05B0Dh
; patch banner, 12 characters
	.DB	"NSC800/MSFT "
	

;--------------------------------------------------------------------------------------	
; SIM patch 
; 6CE1H	(F3H)	DI
; 6CE2H	(3EH)	MVI A,19H	; 0001 1 0 0 1 - there is no 7.5 flip flop to reset, masking BCR
; 6CE4H	(30H)	SIM		


	.org	06CE2h
	call	patch09

	
;--------------------------------------------------------------------------------------
; SIM patch - print to printer
; 6D67H	(3EH)	MVI A,09H
; 6D69H	(30H)	SIM

	.org	06D67h
	call	patch09

;--------------------------------------------------------------------------------------	
; SIM patch - write to cassette
; 6F79H	(3EH)	MVI A,D0H
; 6F7BH	(30H)	SIM

; not supported
	.org	06F79h
	nop
	nop
	nop

;--------------------------------------------------------------------------------------	
; SIM patch - write to cassette
; 6F80H	(3EH)	MVI A,50H
; 6F82H	(30H)	SIM
; not supported

	.org	06F80h
	nop
	nop
	nop
	
;--------------------------------------------------------------------------------------	
; RIM patch - read cassette
; 6FE7H	(D8H)	RC
; 6FE8H	(20H)	RIM
; 6FE9H	(07H)	RLC
; not supported
; need to stub out cassette
	.org 	06FE8H
	nop
;--------------------------------------------------------------------------------------	
; RIM patch - read cassette
; 6FF2H	(20H)	RIM
; 6FF3H	(07H)	RLC
; not supported
; need to stub out cassette
	.org 	06FF2H
	nop
;--------------------------------------------------------------------------------------	
; RIM patch - read from cassette
; 6FFDH	(D8H)	RC
; 6FFEH	(20H)	RIM
; 6FFFH	(07H)	RLC
; not supported
; need to stub out cassette

	.org 	06FFEH
	nop
;--------------------------------------------------------------------------------------	
; RIM patch	
; 7008H	(20H)	RIM
; 7009H	(07H)	RLC
; not supported
; need to stub out cassette	
	.org 	07008H
	nop
		
;--------------------------------------------------------------------------------------
; SIM patch - reset to nominal 
; 71F4H	(3EH)	MVI A,09H
; 71F6H	(30H)	SIM

	.org	071F4h
	call	patch09
	
;--------------------------------------------------------------------------------------
; SIM patch
; 726BH	(3EH)	MVI A,09H
; 726DH	(30H)	SIM
	
	.org	0726Bh
	call	patch09	
		
;--------------------------------------------------------------------------------------
; SIM patch	
; 73E8H	(3EH)	MVI A,09H
; 73EAH	(30H)	SIM

	.org	073E8h
	call	patch09	

;--------------------------------------------------------------------------------------
; SIM patch
; 743CH	(3EH)	MVI A,09H
; 743EH	(30H)	SIM

	.org	0743Ch
	call	patch09	



;--------------------------------------------------------------------------------------
; 75AB to 7640 free to use
	.org	075B0h

;Basic function vector table for SGN to MID$
; moved from 0040h

basfun:
	.DW	3407H,3654H,33F2H,2B4CH
	.DW	1100H,10C8H,10CEH,305AH
	.DW	313EH,2FCFH,30A4H,2EEFH
	.DW	2F09H,2F58H,2F71H,1284H
	.DW	1889H,506DH,506BH,3501H
	.DW	352AH,35BAH,3645H,2943H
	.DW	273AH,2A07H,294FH,295FH
	.DW	298EH,29ABH,29DCH,29E6H



;--------------------------------------------------------------------------------------	
; SIM patch - set interrupt to 1D - warm restart
; 765CH	(F3H)	DI
; 765DH	(3EH)	MVI A,1DH
; 765FH	(30H)	SIM
; 7660H	(FBH)	EI
; 7661H	(C9H)	RET

; treat the same as 0D

	.org	0765Dh
	call	patch1D   
	
;--------------------------------------------------------------------------------------
	.org	07FBFh
; change banner	, 20 char
	.DB	"NSC800 MOD for M100 "


	
; SIM patches:

;--------------------------------------------------------------------------------------
; 1B39H	(3EH)	MVI A,0DH		; 0000   1  1  0  1 -- mask set enable, mask 7.5 and 5.5, enable 6.5
; 1B3BH	(30H)	SIM			;       MSE 75 65 55

; purpose:

; NSC800 uses IO port BB to set mask for interpts
; x x x x A B C I, where the order is
; /RSTA /RSTB /RSTC /INTR, and a 1 enables the interrupt
;   7.5   6.5   5.5   NA

; so this command should be
; mvi	a,00000100
; out	BB


; is there a conflict with BB?  appears to be no.

; 1) need a routine to set mask to equiv of OD and return
; 3 places
; 1B39H	(3EH)	MVI A,0DH	; call routine to set mask 
; 1B3BH	(30H)	SIM
; 1B3CH	(FBH)	EI

; 457BH	(3EH)	MVI A,0DH
; 457DH	(30H)	SIM
; 457EH	(FBH)	EI

; 765CH	(F3H)	DI
; 765DH	(3EH)	MVI A,1DH	; 0 0 0 1 1 1 0 1 
; 765FH	(30H)	SIM		; like above but no flip flop to reset
; 7660H	(FBH)	EI		; this says clear pending interrupt AND mask 7.5
; 7661H	(C9H)	RET		; does not matter for this use case
; 				; treat the same as 0D


; ; 2) replace all these with nop nop nop
; 4582H	(3EH)	MVI A,09H	; 0000 1 0 0 1 -- mask only 5.5.  not used, stubbe out
; 4584H	(30H)	SIM		, nop, nop nop

; 6D67H	(3EH)	MVI A,09H
; 6D69H	(30H)	SIM

; 71F4H	(3EH)	MVI A,09H
; 71F6H	(30H)	SIM

; 726BH	(3EH)	MVI A,09H
; 726DH	(30H)	SIM

; 73E8H	(3EH)	MVI A,09H
; 73EAH	(30H)	SIM

; 743CH	(3EH)	MVI A,09H
; 743EH	(30H)	SIM
	


; 6CE1H	(F3H)	DI
; 6CE2H	(3EH)	MVI A,19H	; 0001 1 0 0 1 - there is no 7.5 flip flop to reset, masking BCR
; 6CE4H	(30H)	SIM		; nop nop nop
; 				; BCR anyways, stub this out


; 6F79H	(3EH)	MVI A,D0H	; 1 1 0 1 0 0 0 0;  send out data 
; 6F7BH	(30H)	SIM		; nop nop nop
; 				; stub out

; 6F80H	(3EH)	MVI A,50H	; 0 1 0 1 0 0 0 0; send out data
; 6F82H	(30H)	SIM		; nop nop nop
; 				; stub out



; RIM patches

; 	6FE8H	(20H)	RIM
; 	6FF2H	(20H)	RIM
; 	6FFEH	(20H)	RIM
; 	7008H	(20H)	RIM

; all deal with cassette
; nop everywhere.

.end

; EI locations

; 0473H	(FBH)	EI

; 
; 19A4H	(F3H)	DI
; 19A5H	(CDH)	CALL 7329H     ; Copy clock chip regs to M
; 19A8H	(FBH)	EI		; hit all the time following 743C 
; 19A9H	(E1H)	POP H
; this one is ok


; 19B6H	(F3H)	DI
; 19B7H	(CDH)	CALL 732AH     ; Update clock chip regs from M
; 19BAH	(FBH)	EI
; 19BBH	(E1H)	POP H
; 19BCH	(C9H)	RET
; ok


; 1B38H	(F5H)	PUSH PSW
; 1B39H	(3EH)	MVI A,0DH
; 1B3BH	(30H)	SIM
; 1B3CH	(FBH)	EI		; hit all the time  RST75
; not ok, purposefully disabling timer


; 3FA0H	(F3H)	DI
; 3FA1H	(7EH)	MOV A,M
; 3FA2H	(E6H)	ANI 04H
; 3FA4H	(F6H)	ORI 01H
; 3FA6H	(BEH)	CMP M
; 3FA7H	(77H)	MOV M,A
; 3FA8H	(CAH)	JZ 3FB0H
; 3FABH	(E6H)	ANI 04H
; 3FADH	(C2H)	JNZ 3FE8H
; 3FB0H	(FBH)	EI
; 3FB1H	(C9H)	RET


; 3FB9H	(F3H)	DI
; 3FBAH	(7EH)	MOV A,M
; 3FBBH	(F5H)	PUSH PSW
; 3FBCH	(F6H)	ORI 02H
; 3FBEH	(77H)	MOV M,A
; 3FBFH	(F1H)	POP PSW
; 3FC0H	(EEH)	XRI 05H
; 3FC2H	(CAH)	JZ 3FFCH
; 3FC5H	(FBH)	EI
; 3FC6H	(C9H)	RET

; 3FC7H	(F3H)	DI
; 3FC8H	(7EH)	MOV A,M
; 3FC9H	(E6H)	ANI 05H
; 3FCBH	(BEH)	CMP M
; 3FCCH	(77H)	MOV M,A
; 3FCDH	(C2H)	JNZ 3FE1H
; 3FD0H	(FBH)	EI
; 3FD1H	(C9H)	RET

; 3FD2H	(F3H)	DI
; 3FD3H	(7EH)	MOV A,M
; 3FD4H	(E6H)	ANI 01H
; 3FD6H	(CAH)	JZ 3FE6H
; 3FD9H	(7EH)	MOV A,M
; 3FDAH	(F6H)	ORI 04H
; 3FDCH	(BEH)	CMP M
; 3FDDH	(CAH)	JZ 3FE6H
; 3FE0H	(77H)	MOV M,A
; 3FE1H	(EEH)	XRI 05H
; 3FE3H	(CAH)	JZ 3FE8H
; 3FE6H	(FBH)	EI
; 3FE7H	(C9H)	RET


; 3FE8H	(3AH)	LDA F654H
; 3FEBH	(3CH)	INR A
; 3FECH	(32H)	STA F654H
; 3FEFH	(FBH)	EI		
; 3FF0H	(C9H)	RET


; 3FF1H	(F3H)	DI
; 3FF2H	(7EH)	MOV A,M
; 3FF3H	(E6H)	ANI 03H
; 3FF5H	(BEH)	CMP M
; 3FF6H	(77H)	MOV M,A
; 3FF7H	(C2H)	JNZ 3FFCH
; 3FFAH	(FBH)	EI		; 
; 3FFBH	(C9H)	RET


; 3FFCH	(3AH)	LDA F654H
; 3FFFH	(D6H)	SUI 01H
; 4001H	(DAH)	JC 3FFAH
; 4004H	(32H)	STA F654H
; 4007H	(FBH)	EI			; not hit during menu
; 4008H	(C9H)	RET



; 457AH	(F3H)	DI
; 457BH	(3EH)	MVI A,0DH
; 457DH	(30H)	SIM
; 457EH	(FBH)	EI		; not hit during menu



; 540BH	(F3H)	DI
; 540CH	(E6H)	ANI 0FH
; 540EH	(4FH)	MOV C,A
; 540FH	(C2H)	JNZ 5414H
; 5412H	(0EH)	MVI C,0AH
; 5414H	(3AH)	LDA F62BH      ; Dial speed (1=10pps, 2=20pps
; 5417H	(0FH)	RRC
; 5418H	(11H)	LXI D,161CH
; 541BH	(D2H)	JNC 5421H
; 541EH	(11H)	LXI D,2440H
; 5421H	(CDH)	CALL 52C1H
; 5424H	(CDH)	CALL 5326H
; 5427H	(1DH)	DCR E
; 5428H	(C2H)	JNZ 5424H
; 542BH	(CDH)	CALL 52B4H     ; Go off-hook
; 542EH	(CDH)	CALL 5326H
; 5431H	(15H)	DCR D
; 5432H	(C2H)	JNZ 542EH
; 5435H	(0DH)	DCR C
; 5436H	(C2H)	JNZ 5414H
; 5439H	(FBH)	EI


; 59FEH	(F3H)	DI
; 59FFH	(CDH)	CALL 73A9H     ; Blink the cursor
; 5A02H	(FBH)	EI		; hit on exit from menu
; 5A03H	(D1H)	POP D


; 6CE1H	(F3H)	DI
; 6CE2H	(3EH)	MVI A,19H	; NOP  NOP  NOP
; 6CE4H	(30H)	SIM
; 6CE5H	(DBH)	IN C8H
; 6CE7H	(3EH)	MVI A,43H
; 6CE9H	(D3H)	OUT B8H
; 6CEBH	(3EH)	MVI A,05H
; 6CEDH	(CDH)	CALL 7383H     ; Set clock chip mode
; 6CF0H	(3EH)	MVI A,EDH
; 6CF2H	(D3H)	OUT BAH
; 6CF4H	(AFH)	XRA A
; 6CF5H	(32H)	STA FF45H      ; Contents of port E8H
; 6CF8H	(D3H)	OUT E8H
; 6CFAH	(D3H)	OUT A8H
; 6CFCH	(CDH)	CALL 7682H     ; Check for optional external controller
; 6CFFH	(CDH)	CALL 7533H
; 6D02H	(AFH)	XRA A
; 6D03H	(D3H)	OUT FEH
; 6D05H	(CDH)	CALL 7533H
; 6D08H	(3EH)	MVI A,3BH
; 6D0AH	(D3H)	OUT FEH
; 6D0CH	(CDH)	CALL 752BH
; 6D0FH	(CDH)	CALL 7533H
; 6D12H	(3EH)	MVI A,39H
; 6D14H	(D3H)	OUT FEH
; 6D16H	(FBH)	EI		; intercept here and einable interrupts and call 760Ah
				; not hit during menu
; 6D17H	(CDH)	CALL 76A0H
; 6D1AH	(D2H)	JNC 6D1EH

;  problem here



; 6F09H	(F3H)	DI
; 6F0AH	(DBH)	IN D8H
; 6F0CH	(E6H)	ANI 01H
; 6F0EH	(57H)	MOV D,A
; 6F0FH	(CDH)	CALL 6ED6H
; 6F12H	(FAH)	JM 6F1AH
; 6F15H	(AAH)	XRA D
; 6F16H	(57H)	MOV D,A
; 6F17H	(CDH)	CALL 6ED6H
; 6F1AH	(FBH)	EI		; not hit during menu


; 6F31H	(3EH)	MVI A,AFH
; 6F33H	(F3H)	DI
; 6F34H	(32H)	STA FF42H      ; XON/XOFF enable flag
; 6F37H	(FBH)	EI		; not hit during menu
; 6F38H	(C9H)	RET


; 71F7H	(F1H)	POP PSW
; 71F8H	(C1H)	POP B
; 71F9H	(D1H)	POP D
; 71FAH	(E1H)	POP H
; 71FBH	(FBH)	EI		; constantly hit during menu, following 71F4
; 71FCH	(C9H)	RET


; 72C5H	(F3H)	DI
; 72C6H	(7BH)	MOV A,E
; 72C7H	(D3H)	OUT BCH
; 72C9H	(7AH)	MOV A,D
; 72CAH	(F6H)	ORI 40H
; 72CCH	(D3H)	OUT BDH
; 72CEH	(3EH)	MVI A,C3H
; 72D0H	(D3H)	OUT B8H
; 72D2H	(DBH)	IN BAH
; 72D4H	(E6H)	ANI F8H
; 72D6H	(F6H)	ORI 20H
; 72D8H	(D3H)	OUT BAH
; 72DAH	(CDH)	CALL 729FH     ; Check if SHIFT-BREAK is being pressed
; 72DDH	(D2H)	JNC 72E8H
; 72E0H	(3EH)	MVI A,03H
; 72E2H	(32H)	STA FFEBH
; 72E5H	(C3H)	JMP 72F9H
; 		
; 72E8H	(0EH)	MVI C,64H
; 72EAH	(C5H)	PUSH B
; 72EBH	(0EH)	MVI C,1EH
; 72EDH	(CDH)	CALL 7657H     ; Delay routine - decrement C until zero
; 72F0H	(C1H)	POP B
; 72F1H	(0DH)	DCR C
; 72F2H	(C2H)	JNZ 72EAH
; 72F5H	(05H)	DCR B
; 72F6H	(C2H)	JNZ 72DAH
; 72F9H	(DBH)	IN BAH
; 72FBH	(F6H)	ORI 04H
; 72FDH	(D3H)	OUT BAH
; 72FFH	(CDH)	CALL 6E83H
; 7302H	(FBH)	EI			; never hit during menu
; 7303H	(C9H)	RET


; 765CH	(F3H)	DI
; 765DH	(3EH)	MVI A,1DH		; constantly hit during Menu
; 765FH	(30H)	SIM
; 7660H	(FBH)	EI			; constantly hit in basic looksl like timer
; 7661H	(C9H)	RET
; problem here

; F5F9H	(FBH)	EI	BCR hook location
; F5FAH	(C9H)	RET	BCR hook location
; F5FBH	(00H)	NOP	BCR hook location
